<?php
/**
 * Classes helpdeskEvent, helpdeskTicket, helpdeskFollowup
 * @package helpdesk
 * Copyright OSI 2005. Licensed under GPL version 2.
 * $Id: event.php,v 1.1 2006/01/17 19:55:10 fgm Exp $
 */

// echo "<p>Start of " . __FILE__ . "</p>" ;

$_event_saved_er  = error_reporting(E_ALL || E_STRICT) ;

/**
 * Import time-related code
 */
require_once("misc.php")     ;
require_once("timeinfo.php") ;

/**
 * import HD core
 */
require_once("core.php");

/**
 * Support event
 * @package helpdesk
 */
abstract class helpdeskEvent extends helpdeskObject 
  {
  public $nid ; ///< link to drupal node object
  
  /**
   * Generates the code for hook_form when node is a Helpdesk event
   * @return string
   */
  function form ()
    {
    // Now we define the form elements specific to our node type. 
    $ret = t ("<p>Defining new %nodename</p>\n", array ('%nodename' => helpdesk_node_name ($node))) ;
    // This would allow format selection, but events are plain text, so we use 1 = filtered html 
    //  $ret .= filter_form ('format', $node->format); 
    $node->format = 1 ; // Filtered HTML
  
    /**
    * TimeID information for new event
    */
    $ret .= helpdeskTimeInfo::form (
      t ('Date and time problem was notified'),
      t('The start time defines the start of any contractual answering answer time guarantee. '
       . 'It is automatically inserted at the time the issue is created and the '
       . 'ticket is issued.'),
      format_date(time(), 'small', NULL,NULL), TRUE,  FALSE,
      format_date(time(), 'small', NULL,NULL), FALSE, FALSE,
      format_date(time(), 'small', NULL,NULL), FALSE, FALSE,
      '', helpdesk_access (HELPDESKHOOKACCESSUPDATE,HELPDESKNODETICKET), TRUE
      ) ;

    /**
     * Contents for new event
     */
    $ret .= form_textarea (
      t ('Describe the issue you\'re having as thouroughly as possible, including the version numbers of any piece of software'
        . ' and the serial number of any product affected by the issue. More precision yields shorter response times.'), 
      'body', isset ($node->body) ? $node->body : '', 60, 20) ;
    return $ret ;
    }
   
  /**
   * Generates a human-readable ticket number based on the nid of the
   * ticket itself, or the ticket to which a followup is bound
   * @return string the readable ticket number
   */ 
  abstract function getTicket () ;
  
  /**
   * @return string HTML
   *
   * @return unknown
   */
  function view()
    {
    return "helpdeskEvent/view(nid = $this->nid)" ;
    }
    
  function init($node, $fromDB)
    {
    if (HELPDESKDEBUGALLFUNCTIONS == TRUE)
      { 
      echo "<pre>HD EVENT/init (" 
        . (isset($node->nid) ? $node->nid : 'new node') 
        . ',' 
        . ($fromDB ? 'from DB' : 'from passed node') 
        . ")</pre>\r\n" ; 
      }
    $this->nid = $node->nid ;
    return get_class($this) . "/view(nid = $this->nid" ;
    }

  /**
   * class constructor
   */
  function __construct()
   {
   $this->nid = 0 ;
   }
  }

/**
 * @package helpdesk
 */
class helpdeskTicket extends helpdeskEvent
  {
  public $timeid ; 
  public $contractnid ;
  public $txnid ;
  public $tidseverity ;
  public $tidstatus ;
  public $tempcontact ;
  public $attachment ;
  public $uidtech ;
  public $techname;
  
  /**
   * @return string the readable ticket number
   */
  function getTicket () 
    {
    $prefix = variable_get(HELPDESKVARTICKETPREFIX, '') ;
    $offset = variable_get(HELPDESKVARTICKETOFFSET , 0)  ;
    
    if (!isset($this->nid))
      return '' ;
    else
      return sprintf ("%s%u", $prefix, $offset + $this->nid) ;
    }

  /**
   * Generates the code for hook_form when node is a helpdesk Ticket
   * @return string HTML
   */
  function form()
    {
    // Now we define the form elements specific to our node type. 
    if (HELPDESKDEBUGALLFUNCTIONS == TRUE)
      { 
      $ret = t ("<p>Defining new %nodename</p>\n", array ('%nodename' => $this->node_name())) ; 
      }
    else 
      $ret = '' ;
      
    
    /* This would allow format selection, but tickets are plain text (are they ?), so we use 1 = filtered html */
    //  $ret .= filter_form ('format', $node->format); 
    // $node->format = 1 ; // Filtered HTML
    // But there's no "node" code in this method ! 
    // @todo find where input filtering goes in this model

    // Select contract holder
    /* To be processed:
    attachment
    contractid
    nidparent, nidroot
    tempcontact
    X ticket
    X tidseverity
    X tidstatus
    txnid
    uidtech
    uiduser
    */
      
    $ret .= t('The human-readable ticket number is ') . $this->getTicket() ;
    
    /* display ticket timeinfo */
    $ti = new helpdeskTimeInfo();
    $ti->init(array(
      'S' => time(),
      'A' => time(),
      'M' => time(),
      'E' => '',
      'showS'    => TRUE, 'showA'    => TRUE,  'showM'    => TRUE,  'showE'    => TRUE,
      'enabledS' => TRUE, 'enabledA' => FALSE, 'enabledM' => FALSE, 'enabledE' => TRUE),
      FALSE) ; // Load from params, not from DB
    $items = $ti->form();
    $ret .= form_group 
      (
      t ('Ticket date information'), 
      $items, 
      t ('Here you must define the date the ticket is issued (default to current date)'
        . ' optionally define when it ends if it happens to be closed during the opening call,'
        . ' and access/modification dates (default to current date)'
        ), 
      NULL) ;

    //==== OK above this line ====  
    /* this might be usable for terms selection, although a better function probably exists in taxonomy.module  */
    $arTerms = _helpdesk_get_terms(HELPDESKVARVIDSEVERITY) ;
    $ret .= form_select(
      t('Severity of event'),         // title
      HELPDESKVARVIDSEVERITY,                           // name
      variable_get(HELPDESKVARVIDSEVERITY, NULL),       // current value
      $arTerms,                                          // array of vocabularies
      t('Choose the severity from the values allowed for severity conditions.'),  // description
      NULL,                                             // extra
      FALSE,                                            // Only one vocabulary used
      TRUE);                                            // One vocabulary must be selected
/*    */

    /**
     * Parameters for new ticket
     */
    $items = form_checkbox (t ('Suspended'), 'isSuspended', 1, $this->isSuspended, '', NULL, FALSE) ;
    $items .= theme_table 
      (
      array (t ('Contract limits'), t ('Limit value'), t ('Increment')),
      array 
        (
        array  (
          form_checkbox (t ('Minutes spent'),      'useMaxTime',   1,  $this->useMaxTime, '', NULL, FALSE),
          form_textfield ('',                      'maxTime',          $this->maxTime, 16, 16, NULL, NULL, FALSE),
          form_textfield ('',                      'defTimeIncrement', $this->defTimeIncrement, 4, 4, NULL, NULL, FALSE)
	      ),
        array (
          form_checkbox (t ('Number of tickets'),  'useMaxIssue',  1,  $this->useMaxIssue, '', NULL, FALSE),
          form_textfield ('',                      'maxIssue',         $this->maxIssue, 16, 16, NULL, NULL, FALSE),
          '&nbsp;'),
        array (
          form_checkbox (t ('Date'),               'useMaxDate',   1,  $this->useMaxDate, '', NULL, FALSE),
          form_textfield ('',                      'maxDate',          $this->maxDate, 16, 16, NULL, NULL, FALSE),
	      '&nbsp;')
        ) 
      ) ;
  
    $ret .= form_group (
      t ('Contract status'),
      $items, 
      t ('Here you set the attributes of the contract: is it time-limited, duration-limited, issue-limited ? does it start suspended ?'
        . ' The increment value for time-limited contracts is the default duration consumed by each followup.'),
      NULL) ;

    $ret .= form_textarea(t('Contract wording'), 'body', (isset($this->node->body) ? $this->node->body : ''), 60, 20); 
    return $ret ;  
    }
  
function init($node, $fromDB)
    {
    if (HELPDESKDEBUGALLFUNCTIONS == TRUE)
      { echo "<pre>HD TICKET/init (" . (isset($node->nid) ? $node->nid : 'new node') . ", fromDB = " . ($fromDB ? 'True' : 'False') . ")</pre>\r\n" ; }
    $ret = parent::init($node, $fromDB);
    if ($fromDB)    
      {
      // As we are querying for a Ticket, parent and root will be equal to self: we don't need to query for them
      $q = '
        SELECT 
          hde.timeid,       hde.contractnid,    hde.txnid, 
          hde.tidseverity,  hde.tidstatus,
          hde.tempcontact,  hde.attachment,
          hdt.stime,        hdt.atime,          hdt.mtime,      hdt.etime,
          hde.uidtech,      u.name techname
        FROM {hd_event} hde
          LEFT JOIN {hd_timeinfo} hdt on hde.timeid = hdt.timeid
          LEFT JOIN {users} u on hde.uidtech = u.uid
        WHERE 
          hde.nid = %d
        '  ; 
      $req = db_query ($q, $node->nid);
      $ar = db_fetch_array($req) ; // Should give exactly one row of result, zero if ticket does not exist yet
      foreach ($ar as $key => $value) 
        {
        $this->$key = $value ; 	
        } 
      echo "<pre>New ticket object: " . print_r ($this, True) . "</pre>\r\n";
      }
    else  
      echo "<p>Ticket not from DB</p>" ;
    }
    
  /**
   * @return string HTML
   *
   */
  function view()
    {
    $headers = array(
      t('Parameter'), 
      t('Value')
      );

    $cnode = node_load(array('nid' => $this->contractnid));
    // Must still add display for timeid, txnid, tempcontact, attachment
    $rows = array
      ( // Row with attributes on the row and some of its cells.   
      array 
        (
        t('Current ticket status'), 
        array ('data' => taxonomy_get_term ($this->tidstatus)->name,   'style' => 'text-align: center')
        ),
      array 
        (
        t('Current ticket severity'), 
        array ('data' => taxonomy_get_term ($this->tidseverity)->name, 'style' => 'text-align: center')
        ),
      array 
        (
        t('Helpdesk Tech in charge'), 
        array ('data' => isset ($this->techname) ? l($this->techname, "user/$this->uidtech") : t('None assigned yet'), 'style' => 'text-align: center'),
        ),
      array 
        (
        t('Contract being charged'), 
        array ('data' => l($cnode->title, "node/$this->contractnid"), 'style' => 'text-align: center'),
        )
      );
    
    $output = theme_table($headers, $rows);
    return $output;
    }
  }
  
/**
 * @package helpdesk
 */
class helpdeskFollowup extends helpdeskTicket 
  {
  }
  
  
error_reporting($_event_saved_er) ;
unset($_event_saved_er) ;
// echo "<p>End of " . __FILE__ . "</p>" ;
?>
